@@ -233,7 +233,7 @@ static PyTypeObject PyGInterface_Type = {
 
 static void
 pyg_register_interface(PyObject *dict, const gchar *class_name,
-		       GType gtype, PyTypeObject *type)
+                       GType gtype, PyTypeObject *type)
 {
     PyObject *o;
 
@@ -241,19 +241,260 @@ pyg_register_interface(PyObject *dict, const gchar *class_name,
     type->tp_base = &PyGInterface_Type;
 
     if (PyType_Ready(type) < 0) {
-	g_warning("could not ready `%s'", type->tp_name);
-	return;
+        g_warning("could not ready `%s'", type->tp_name);
+        return;
     }
 
     if (gtype) {
-	o = pyg_type_wrapper_new(gtype);
-	PyDict_SetItemString(type->tp_dict, "__gtype__", o);
-	Py_DECREF(o);
+        o = pyg_type_wrapper_new(gtype);
+        PyDict_SetItemString(type->tp_dict, "__gtype__", o);
+        Py_DECREF(o);
     }
 
     PyDict_SetItemString(dict, (char *)class_name, (PyObject *)type);
 }
 
+/* -------------- GMainContext objects ---------------------------- */
+
+static int
+pyg_main_context_new(PyGMainContext *self)
+{
+    self->context = g_main_context_new();
+    return 0;
+}
+
+static int
+pyg_main_context_compare(PyGMainContext *self, PyGMainContext *v)
+{
+    if (self->context == v->context) return 0;
+    if (self->context > v->context) return -1;
+    return 1;
+}
+
+static PyObject *
+pyg_main_context_default (PyObject *unused)
+{
+    PyGMainContext *self;
+
+    self = (PyGMainContext *)PyObject_NEW(PyGMainContext,
+					  &PyGMainContext_Type);
+    if (self == NULL)
+	return NULL;
+
+    self->context = g_main_context_default();
+    return (PyObject *)self;
+
+}
+
+static PyObject *
+_wrap_g_main_context_iteration (PyGMainContext *self, PyObject *args)
+{
+    PyObject *py_ret;
+    gboolean may_block = TRUE;
+    
+    if (!PyArg_ParseTuple(args, "|b:GMainContext.iteration",
+			  &may_block))
+	return NULL;
+	
+    py_ret = g_main_context_iteration(self->context, may_block)
+	? Py_True : Py_False;
+    
+    Py_INCREF(py_ret);
+    return py_ret;
+}
+
+static PyObject *
+_wrap_g_main_context_pending (PyGMainContext *self)
+{
+    PyObject *py_ret;
+
+    py_ret = g_main_context_pending(self->context) ? Py_True : Py_False;
+    
+    Py_INCREF(py_ret);
+    return py_ret;
+}
+
+static PyMethodDef _PyGMainContext_methods[] = {
+    { "iteration", (PyCFunction)_wrap_g_main_context_iteration, METH_VARARGS },
+    { "pending", (PyCFunction)_wrap_g_main_context_pending, METH_NOARGS },
+    { NULL, NULL, 0 }
+};
+
+PyTypeObject PyGMainContext_Type = {
+    PyObject_HEAD_INIT(NULL)
+    0,			
+    "gobject.MainContext",	
+    sizeof(PyGMainContext),	
+    0,			
+    /* methods */
+    (destructor)0,	
+    (printfunc)0,	
+    (getattrfunc)0,	
+    (setattrfunc)0,	
+    (cmpfunc)pyg_main_context_compare,		
+    (reprfunc)0,	
+    0,			
+    0,		
+    0,		
+    (hashfunc)0,		
+    (ternaryfunc)0,		
+    (reprfunc)0,		
+    (getattrofunc)0,		
+    (setattrofunc)0,		
+    0,				
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, 
+    NULL,
+    (traverseproc)0,		
+    (inquiry)0,
+    (richcmpfunc)0,	
+    0,             
+    (getiterfunc)0,
+    (iternextfunc)0,
+    _PyGMainContext_methods,
+    0,				
+    0,		       	
+    NULL,		
+    NULL,		
+    (descrgetfunc)0,	
+    (descrsetfunc)0,	
+    0,                 
+    (initproc)pyg_main_context_new,
+};
+
+/* -------------- GMainLoop objects ---------------------------- */
+
+static int
+pyg_main_loop_new(PyGMainLoop *self, PyObject *args, PyObject *kwargs)
+{
+
+    static char *kwlist[] = { "context", "is_running", NULL };
+    PyObject *py_context = Py_None;
+    int is_running;
+    GMainContext *context;
+    
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs,
+				     "|Ob:GMainLoop.__init__",
+				     kwlist, &py_context, &is_running))
+        return -1;
+
+    if (!PyObject_TypeCheck(py_context, &PyGMainContext_Type) &&
+	py_context != Py_None) {
+	PyErr_SetString(PyExc_TypeError,
+			"context must be a gobject.GMainContext or None");
+	return -1;
+    }
+
+    if (py_context != Py_None) {
+	context = ((PyGMainContext*)py_context)->context;
+    } else {
+	context = NULL;
+    }
+
+    self->loop = g_main_loop_new(context, is_running);
+    return 0;
+}
+
+static int
+pyg_main_loop_compare(PyGMainLoop *self, PyGMainLoop *v)
+{
+    if (self->loop == v->loop) return 0;
+    if (self->loop > v->loop) return -1;
+    return 1;
+}
+
+static PyObject *
+_wrap_g_main_loop_get_context (PyGMainLoop *loop)
+{
+    PyGMainContext *self;
+
+    self = (PyGMainContext *)PyObject_NEW(PyGMainContext,
+					  &PyGMainContext_Type);
+    
+    self->context = g_main_loop_get_context(loop->loop);
+    
+    if (self->context == NULL)
+	return NULL;
+
+    return (PyObject *)self;
+}
+
+static PyObject *
+_wrap_g_main_loop_is_running (PyGMainLoop *self)
+{
+    PyObject *py_ret;
+    
+    py_ret = g_main_loop_is_running(self->loop) ? Py_True : Py_False;
+    Py_INCREF(py_ret);
+    return py_ret;
+    
+}
+
+static PyObject *
+_wrap_g_main_loop_quit (PyGMainLoop *self)
+{
+    g_main_loop_quit(self->loop);
+    
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *
+_wrap_g_main_loop_run (PyGMainLoop *self)
+{
+    g_main_loop_run(self->loop);
+    
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyMethodDef _PyGMainLoop_methods[] = {
+    { "get_context", (PyCFunction)_wrap_g_main_loop_get_context, METH_NOARGS },
+    { "is_running", (PyCFunction)_wrap_g_main_loop_is_running, METH_NOARGS },
+    { "quit", (PyCFunction)_wrap_g_main_loop_quit, METH_NOARGS },
+    { "run", (PyCFunction)_wrap_g_main_loop_run, METH_NOARGS },
+    { NULL, NULL, 0 }
+};
+
+PyTypeObject PyGMainLoop_Type = {
+    PyObject_HEAD_INIT(NULL)
+    0,			
+    "gobject.MainLoop",	
+    sizeof(PyGMainLoop),	
+    0,			
+    /* methods */
+    (destructor)0,	
+    (printfunc)0,	
+    (getattrfunc)0,	
+    (setattrfunc)0,	
+    (cmpfunc)pyg_main_loop_compare,
+    (reprfunc)0,	
+    0,			
+    0,		
+    0,		
+    (hashfunc)0,		
+    (ternaryfunc)0,		
+    (reprfunc)0,		
+    (getattrofunc)0,		
+    (setattrofunc)0,		
+    0,				
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, 
+    NULL,
+    (traverseproc)0,		
+    (inquiry)0,
+    (richcmpfunc)0,	
+    0,             
+    (getiterfunc)0,
+    (iternextfunc)0,
+    _PyGMainLoop_methods,
+    0,				
+    0,		       	
+    NULL,		
+    NULL,		
+    (descrgetfunc)0,	
+    (descrsetfunc)0,	
+    0,                 
+    (initproc)pyg_main_loop_new,
+};
 
 /* ---------------- gobject module functions -------------------- */
 
@@ -384,8 +625,11 @@ pyg_object_set_property (GObject *object, guint property_id,
     object_wrapper = pygobject_new(object);
     g_return_if_fail(object_wrapper != NULL);
 
+    pyg_block_threads();
+
     py_pspec = pyg_param_spec_new(pspec);
     py_value = pyg_value_as_pyobject (value, TRUE);
+
     retval = PyObject_CallMethod(object_wrapper, "do_set_property",
 				 "OO", py_pspec, py_value);
     if (retval) {
@@ -394,9 +638,12 @@ pyg_object_set_property (GObject *object, guint property_id,
 	PyErr_Print();
 	PyErr_Clear();
     }
+
     Py_DECREF(object_wrapper);
     Py_DECREF(py_pspec);
     Py_DECREF(py_value);
+
+    pyg_unblock_threads();
 }
 
 static void
@@ -409,6 +656,8 @@ pyg_object_get_property (GObject *object, guint property_id,
     object_wrapper = pygobject_new(object);
     g_return_if_fail(object_wrapper != NULL);
 
+    pyg_block_threads();
+
     py_pspec = pyg_param_spec_new(pspec);
     retval = PyObject_CallMethod(object_wrapper, "do_get_property",
 				 "O", py_pspec);
@@ -419,6 +668,8 @@ pyg_object_get_property (GObject *object, guint property_id,
     Py_DECREF(object_wrapper);
     Py_DECREF(py_pspec);
     Py_XDECREF(retval);
+    
+    pyg_unblock_threads();        
 }
 
 static void
@@ -1099,10 +1350,8 @@ pyg_object_new (PyGObject *self, PyObject *args, PyObject *kwargs)
     GType type;
     GObject *obj = NULL;
     GObjectClass *class;
-    PyObject *value;
-    PyObject *key;
-    int pos=0, num_params=0, i;
-    GParameter *params;
+    int n_params = 0, i;
+    GParameter *params = NULL;
 
     if (!PyArg_ParseTuple (args, "O:gobject.new", &pytype)) {
 	return NULL;
@@ -1110,48 +1359,55 @@ pyg_object_new (PyGObject *self, PyObject *args, PyObject *kwargs)
 
     if ((type = pyg_type_from_object (pytype)) == 0)
 	return NULL;
-    
+
+    if (G_TYPE_IS_ABSTRACT(type)) {
+	PyErr_Format(PyExc_TypeError, "cannot create instance of abstract "
+		     "(non-instantiable) type `%s'", g_type_name(type));
+	return NULL;
+    }
+
     if ((class = g_type_class_ref (type)) == NULL) {
 	PyErr_SetString(PyExc_TypeError,
 			"could not get a reference to type class");
 	return NULL;
     }
 
-    params = g_new0(GParameter, PyDict_Size(kwargs));
-    
-    while (kwargs && PyDict_Next (kwargs, &pos, &key, &value)) {
-	GParamSpec *pspec;
-	gchar *key_str = g_strdup(PyString_AsString (key));
-	pspec = g_object_class_find_property (class, key_str);
-	if (!pspec) {
-	    gchar buf[512];
+    if (kwargs) {
+	int pos = 0;
+	PyObject *key;
+	PyObject *value;
 
-	    g_snprintf(buf, sizeof(buf),
-		       "gobject `%s' doesn't support property `%s'",
-		       g_type_name(type), key_str);
-	    PyErr_SetString(PyExc_TypeError, buf);
-	    goto cleanup;
-	}
-	g_value_init(&params[num_params].value,
-		     G_PARAM_SPEC_VALUE_TYPE(pspec));
-	if (pyg_value_from_pyobject(&params[num_params].value, value)) {
-	    gchar buf[512];
+	params = g_new0(GParameter, PyDict_Size(kwargs));
+	while (PyDict_Next (kwargs, &pos, &key, &value)) {
+	    GParamSpec *pspec;
+	    const gchar *key_str = PyString_AsString (key);
 
-	    g_snprintf(buf, sizeof(buf),
-		       "could not convert value for property `%s'", key_str);
-	    PyErr_SetString(PyExc_TypeError, buf);
-	    goto cleanup;
+	    pspec = g_object_class_find_property (class, key_str);
+	    if (!pspec) {
+		PyErr_Format(PyExc_TypeError,
+			     "gobject `%s' doesn't support property `%s'",
+			     g_type_name(type), key_str);
+		goto cleanup;
+	    }
+	    g_value_init(&params[n_params].value,
+			 G_PARAM_SPEC_VALUE_TYPE(pspec));
+	    if (pyg_value_from_pyobject(&params[n_params].value, value)) {
+		PyErr_Format(PyExc_TypeError,
+			     "could not convert value for property `%s'",
+			     key_str);
+		goto cleanup;
+	    }
+	    params[n_params].name = g_strdup(key_str);
+	    n_params++;
 	}
-	params[num_params].name = key_str;
-	num_params++;
     }
 
-    obj = g_object_newv(type, num_params, params);
+    obj = g_object_newv(type, n_params, params);
     if (!obj)
 	PyErr_SetString (PyExc_RuntimeError, "could not create object");
 	   
  cleanup:
-    for (i = 0; i < num_params; i++) {
+    for (i = 0; i < n_params; i++) {
 	g_free((gchar *) params[i].name);
 	g_value_unset(&params[i].value);
     }
@@ -1163,6 +1419,240 @@ pyg_object_new (PyGObject *self, PyObject *args, PyObject *kwargs)
     return NULL;
 }
 
+static gint
+get_handler_priority(gint *priority, PyObject *kwargs)
+{
+    gint len, pos;
+    PyObject *key, *val;
+
+    /* no keyword args? leave as default */
+    if (kwargs == NULL)	return 0;
+
+    len = PyDict_Size(kwargs);
+    if (len == 0) return 0;
+
+    if (len != 1) {
+	PyErr_SetString(PyExc_TypeError,
+			"expecting at most one keyword argument");
+	return -1;
+    }
+    pos = 0;
+    PyDict_Next(kwargs, &pos, &key, &val);
+    if (!PyString_Check(key)) {
+	PyErr_SetString(PyExc_TypeError,
+			"keyword argument name is not a string");
+	return -1;
+    }
+    if (!strcmp(PyString_AsString(key), "priority")) {
+	PyErr_SetString(PyExc_TypeError,
+			"only 'priority' keyword argument accepted");
+	return -1;
+    }
+
+    *priority = PyInt_AsLong(val);
+    if (PyErr_Occurred()) {
+	PyErr_Clear();
+	PyErr_SetString(PyExc_ValueError, "could not get priority value");
+	return -1;
+    }
+    return 0;
+}
+
+static gboolean
+handler_marshal(gpointer user_data)
+{
+    PyObject *tuple, *ret;
+    gboolean res;
+
+    g_return_val_if_fail(user_data != NULL, FALSE);
+
+    pyg_block_threads();
+
+    tuple = (PyObject *)user_data;
+    ret = PyObject_CallObject(PyTuple_GetItem(tuple, 0),
+			      PyTuple_GetItem(tuple, 1));
+    if (!ret) {
+	PyErr_Print();
+	res = FALSE;
+    } else {
+	res = PyObject_IsTrue(ret);
+	Py_DECREF(ret);
+    }
+    pyg_unblock_threads();
+
+    return res;
+}
+
+static PyObject *
+pyg_idle_add(PyObject *self, PyObject *args, PyObject *kwargs)
+{
+    PyObject *first, *callback, *cbargs = NULL, *data;
+    gint len, priority = G_PRIORITY_DEFAULT_IDLE;
+    guint handler_id;
+
+    len = PyTuple_Size(args);
+    if (len < 1) {
+	PyErr_SetString(PyExc_TypeError,
+			"idle_add requires at least 1 argument");
+	return NULL;
+    }
+    first = PySequence_GetSlice(args, 0, 1);
+    if (!PyArg_ParseTuple(first, "O:idle_add", &callback)) {
+	Py_DECREF(first);
+        return NULL;
+    }
+    Py_DECREF(first);
+    if (!PyCallable_Check(callback)) {
+        PyErr_SetString(PyExc_TypeError, "first argument not callable");
+        return NULL;
+    }
+    if (get_handler_priority(&priority, kwargs) < 0)
+	return NULL;
+
+    cbargs = PySequence_GetSlice(args, 1, len);
+    if (cbargs == NULL)
+      return NULL;
+
+    data = Py_BuildValue("(ON)", callback, cbargs);
+    if (data == NULL)
+      return NULL;
+    handler_id = g_idle_add_full(priority, handler_marshal, data,
+				 (GDestroyNotify)pyg_destroy_notify);
+    return PyInt_FromLong(handler_id);
+}
+
+static PyObject *
+pyg_timeout_add(PyObject *self, PyObject *args, PyObject *kwargs)
+{
+    PyObject *first, *callback, *cbargs = NULL, *data;
+    gint len, priority = G_PRIORITY_DEFAULT, interval;
+    guint handler_id;
+
+    len = PyTuple_Size(args);
+    if (len < 2) {
+	PyErr_SetString(PyExc_TypeError,
+			"timeout_add requires at least 2 args");
+	return NULL;
+    }
+    first = PySequence_GetSlice(args, 0, 2);
+    if (!PyArg_ParseTuple(first, "iO:timeout_add", &interval, &callback)) {
+	Py_DECREF(first);
+        return NULL;
+    }
+    Py_DECREF(first);
+    if (!PyCallable_Check(callback)) {
+        PyErr_SetString(PyExc_TypeError, "second argument not callable");
+        return NULL;
+    }
+    if (get_handler_priority(&priority, kwargs) < 0)
+	return NULL;
+
+    cbargs = PySequence_GetSlice(args, 2, len);
+    if (cbargs == NULL)
+      return NULL;
+
+    data = Py_BuildValue("(ON)", callback, cbargs);
+    if (data == NULL)
+      return NULL;
+    handler_id = g_timeout_add_full(priority, interval, handler_marshal, data,
+				    (GDestroyNotify)pyg_destroy_notify);
+    return PyInt_FromLong(handler_id);
+}
+
+static gboolean
+iowatch_marshal(GIOChannel *source, GIOCondition condition, gpointer user_data)
+{
+    PyObject *tuple, *func, *firstargs, *args, *ret;
+    gboolean res;
+
+    g_return_val_if_fail(user_data != NULL, FALSE);
+
+    pyg_block_threads();
+
+    tuple = (PyObject *)user_data;
+    func = PyTuple_GetItem(tuple, 0);
+
+    /* arg vector is (fd, condtion, *args) */
+    firstargs = Py_BuildValue("(Oi)", PyTuple_GetItem(tuple, 1), condition);
+    args = PySequence_Concat(firstargs, PyTuple_GetItem(tuple, 2));
+    Py_DECREF(firstargs);
+
+    ret = PyObject_CallObject(func, args);
+    Py_DECREF(args);
+    if (!ret) {
+	PyErr_Print();
+	res = FALSE;
+    } else {
+	res = PyObject_IsTrue(ret);
+	Py_DECREF(ret);
+    }
+
+    pyg_unblock_threads();
+
+    return res;
+}
+
+static PyObject *
+pyg_io_add_watch(PyObject *self, PyObject *args, PyObject *kwargs)
+{
+    PyObject *first, *pyfd, *callback, *cbargs = NULL, *data;
+    gint fd, priority = G_PRIORITY_DEFAULT, condition, len;
+    GIOChannel *iochannel;
+    guint handler_id;
+
+    len = PyTuple_Size(args);
+    if (len < 3) {
+	PyErr_SetString(PyExc_TypeError,
+			"timeout_add requires at least 3 args");
+	return NULL;
+    }
+    first = PySequence_GetSlice(args, 0, 3);
+    if (!PyArg_ParseTuple(first, "OiO:io_add_watch", &pyfd, &condition,
+			  &callback)) {
+	Py_DECREF(first);
+        return NULL;
+    }
+    Py_DECREF(first);
+    fd = PyObject_AsFileDescriptor(pyfd);
+    if (fd < 0) {
+	return NULL;
+    }
+    if (!PyCallable_Check(callback)) {
+        PyErr_SetString(PyExc_TypeError, "second argument not callable");
+        return NULL;
+    }
+    if (get_handler_priority(&priority, kwargs) < 0)
+	return NULL;
+
+    cbargs = PySequence_GetSlice(args, 3, len);
+    if (cbargs == NULL)
+      return NULL;
+    data = Py_BuildValue("(OON)", callback, pyfd, cbargs);
+    if (data == NULL)
+      return NULL;
+    iochannel = g_io_channel_unix_new(fd);
+    handler_id = g_io_add_watch_full(iochannel, priority, condition,
+				     iowatch_marshal, data,
+				    (GDestroyNotify)pyg_destroy_notify);
+    g_io_channel_unref(iochannel);
+    
+    return PyInt_FromLong(handler_id);
+}
+
+static PyObject *
+pyg_source_remove(PyObject *self, PyObject *args)
+{
+    guint tag;
+    PyObject *ret;
+
+    if (!PyArg_ParseTuple(args, "i:source_remove", &tag))
+	return NULL;
+
+    ret = g_source_remove(tag) ? Py_True : Py_False;
+    Py_INCREF(ret);
+    return ret;
+}
+
 static PyMethodDef pygobject_functions[] = {
     { "type_name", pyg_type_name, METH_VARARGS },
     { "type_from_name", pyg_type_from_name, METH_VARARGS },
@@ -1175,6 +1665,11 @@ static PyMethodDef pygobject_functions[] = {
     { "signal_list_names", pyg_signal_list_names, METH_VARARGS },
     { "list_properties", pyg_object_class_list_properties, METH_VARARGS },
     { "new", (PyCFunction)pyg_object_new, METH_VARARGS|METH_KEYWORDS },
+    { "idle_add", (PyCFunction)pyg_idle_add, METH_VARARGS|METH_KEYWORDS },
+    { "timeout_add", (PyCFunction)pyg_timeout_add, METH_VARARGS|METH_KEYWORDS },
+    { "io_add_watch", (PyCFunction)pyg_io_add_watch, METH_VARARGS|METH_KEYWORDS },
+    { "source_remove", pyg_source_remove, METH_VARARGS },
+    { "main_context_default", (PyCFunction)pyg_main_context_default, METH_NOARGS },
     { NULL, NULL, 0 }
 };
 
@@ -1263,7 +1758,9 @@ pyg_error_check(GError **error)
     if (*error != NULL) {
 	PyObject *exc_instance;
 	PyObject *d;
-
+	
+	pyg_unblock_threads();
+	
 	exc_instance = PyObject_CallFunction(gerror_exc, "z",
 					     (*error)->message);
 	PyObject_SetAttrString(exc_instance, "domain",
@@ -1283,6 +1780,9 @@ pyg_error_check(GError **error)
 	PyErr_SetObject(gerror_exc, exc_instance);
 	Py_DECREF(exc_instance);
 	g_clear_error(error);
+	
+	pyg_unblock_threads();
+	
 	return TRUE;
     }
     return FALSE;
@@ -1387,6 +1887,20 @@ initgobject(void)
 			 o=pyg_type_wrapper_new(G_TYPE_BOXED));
     Py_DECREF(o);
 
+    PyGMainLoop_Type.ob_type = &PyType_Type;
+    PyGMainLoop_Type.tp_alloc = PyType_GenericAlloc;
+    PyGMainLoop_Type.tp_new = PyType_GenericNew;
+    if (PyType_Ready(&PyGMainLoop_Type))
+	return;
+    PyDict_SetItemString(d, "MainLoop", (PyObject *)&PyGMainLoop_Type);
+    
+    PyGMainContext_Type.ob_type = &PyType_Type;
+    PyGMainContext_Type.tp_alloc = PyType_GenericAlloc;
+    PyGMainContext_Type.tp_new = PyType_GenericNew;
+    if (PyType_Ready(&PyGMainContext_Type))
+	return;
+    PyDict_SetItemString(d, "MainContext", (PyObject *)&PyGMainContext_Type);
+
     PyGPointer_Type.ob_type = &PyType_Type;
     PyGPointer_Type.tp_alloc = PyType_GenericAlloc;
     PyGPointer_Type.tp_new = PyType_GenericNew;
@@ -1423,6 +1937,19 @@ initgobject(void)
     PyModule_AddIntConstant(m, "PARAM_LAX_VALIDATION", G_PARAM_LAX_VALIDATION);
     PyModule_AddIntConstant(m, "PARAM_READWRITE", G_PARAM_READWRITE);
 
+    PyModule_AddIntConstant(m, "PRIORITY_HIGH", G_PRIORITY_HIGH);
+    PyModule_AddIntConstant(m, "PRIORITY_DEFAULT", G_PRIORITY_DEFAULT);
+    PyModule_AddIntConstant(m, "PRIORITY_HIGH_IDLE", G_PRIORITY_HIGH_IDLE);
+    PyModule_AddIntConstant(m,"PRIORITY_DEFAULT_IDLE",G_PRIORITY_DEFAULT_IDLE);
+    PyModule_AddIntConstant(m, "PRIORITY_LOW", G_PRIORITY_LOW);
+
+    PyModule_AddIntConstant(m, "IO_IN",   G_IO_IN);
+    PyModule_AddIntConstant(m, "IO_OUT",  G_IO_OUT);
+    PyModule_AddIntConstant(m, "IO_PRI",  G_IO_PRI);
+    PyModule_AddIntConstant(m, "IO_ERR",  G_IO_ERR);
+    PyModule_AddIntConstant(m, "IO_HUP",  G_IO_HUP);
+    PyModule_AddIntConstant(m, "IO_NVAL", G_IO_NVAL);
+
     PyModule_AddObject(m, "TYPE_INVALID", pyg_type_wrapper_new(G_TYPE_INVALID));
     PyModule_AddObject(m, "TYPE_NONE", pyg_type_wrapper_new(G_TYPE_NONE));
     PyModule_AddObject(m, "TYPE_INTERFACE", pyg_type_wrapper_new(G_TYPE_INTERFACE));
