@@ -191,6 +191,9 @@ pygtk_generic_tree_model_get_flags(GtkTreeModel *tree_model)
     PyObject *self, *py_ret;
 
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), 0);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -202,10 +205,12 @@ pygtk_generic_tree_model_get_flags(GtkTreeModel *tree_model)
 	guint ret = PyInt_AsLong(py_ret);
 
 	Py_DECREF(py_ret);
+	pyg_unblock_threads();
 	return ret;
     } else {
 	PyErr_Print();
 	PyErr_Clear();
+	pyg_unblock_threads();
 	return 0;
     }
 }
@@ -217,6 +222,9 @@ pygtk_generic_tree_model_get_n_columns(GtkTreeModel *tree_model)
 
     g_return_val_if_fail(tree_model != NULL, 0);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), 0);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -228,10 +236,12 @@ pygtk_generic_tree_model_get_n_columns(GtkTreeModel *tree_model)
 	gint ret = PyInt_AsLong(py_ret);
 
 	Py_DECREF(py_ret);
+	pyg_unblock_threads();
 	return ret;
     } else {
 	PyErr_Print();
 	PyErr_Clear();
+	pyg_unblock_threads();
 	return 0;
     }
 }
@@ -243,6 +253,8 @@ pygtk_generic_tree_model_get_column_type(GtkTreeModel *tree_model, gint index)
 
     g_return_val_if_fail(tree_model != NULL, G_TYPE_INVALID);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), G_TYPE_INVALID);
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -255,10 +267,12 @@ pygtk_generic_tree_model_get_column_type(GtkTreeModel *tree_model, gint index)
 	GType ret = pyg_type_from_object(py_ret);
 
 	Py_DECREF(py_ret);
+	pyg_unblock_threads();
 	return ret;
     } else {
 	PyErr_Print();
 	PyErr_Clear();
+	pyg_unblock_threads();
 	return G_TYPE_INVALID;
     }
 }
@@ -273,11 +287,14 @@ pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     g_return_val_if_fail(path != NULL, FALSE);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
 #ifdef DEBUG_TREE_MODEL
-    g_message("get_tree_iter(%p)", path);
+    g_message("get_iter(%p)", path);
 #endif
     py_path = pygtk_tree_path_to_pyobject(path);
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_iter",
@@ -290,16 +307,19 @@ pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
+	    pyg_unblock_threads();
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
+	    pyg_unblock_threads();
 	    return FALSE;
 	}
     } else {
 	PyErr_Print();
 	PyErr_Clear();
 	iter->user_data = NULL;
+	pyg_unblock_threads();
 	return FALSE;
     }
 }
@@ -316,21 +336,24 @@ pygtk_generic_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
     self = pygobject_new((GObject *)tree_model);
 
 #ifdef DEBUG_TREE_MODEL
-    g_message("get_tree_path(%p)", iter);
+    g_message("get_path(%p)", iter);
 #endif
-    py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_tree_path",
+    pyg_block_threads();
+    py_ret = PyObject_CallMethod(self, METHOD_PREFIX "get_path",
 				 "(O)", (PyObject *)iter->user_data);
     if (py_ret) {
 	GtkTreePath *path = pygtk_tree_path_from_pyobject(py_ret);
 
 	if (!path)
-	    g_warning("could not convert return value of get_tree_path() to "
+	    g_warning("could not convert return value of get_path() to "
 		      "a GtkTreePath");
 	Py_DECREF(py_ret);
+	pyg_unblock_threads();
 	return path;
     } else {
 	PyErr_Print();
 	PyErr_Clear();
+	pyg_unblock_threads();
 	return NULL;
     }
 }
@@ -344,6 +367,9 @@ pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
     g_return_if_fail(tree_model != NULL);
     g_return_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model));
     g_return_if_fail(iter != NULL);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -364,6 +390,7 @@ pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
 	PyErr_Print();
 	PyErr_Clear();
     }
+    pyg_unblock_threads();
 }
 
 static gboolean
@@ -374,6 +401,9 @@ pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -389,16 +419,19 @@ pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
+	    pyg_unblock_threads();
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
+	    pyg_unblock_threads();
 	    return FALSE;
 	}
     } else {
 	iter->user_data = NULL;
 	PyErr_Print();
 	PyErr_Clear();
+	pyg_unblock_threads();
 	return FALSE;
     }
 }
@@ -412,6 +445,9 @@ pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model, GtkTreeIter *it
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -429,16 +465,19 @@ pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model, GtkTreeIter *it
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
+	    pyg_unblock_threads();
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
+	    pyg_unblock_threads();
 	    return FALSE;
 	}
     } else {
 	iter->user_data = NULL;
 	PyErr_Print();
 	PyErr_Clear();
+	pyg_unblock_threads();
 	return FALSE;
     }
 }
@@ -451,6 +490,9 @@ pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model, GtkTreeIter *i
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -463,10 +505,12 @@ pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model, GtkTreeIter *i
 	gboolean ret = PyObject_IsTrue(py_ret);
 
 	Py_DECREF(py_ret);
+	pyg_unblock_threads();
 	return ret;
     } else {
 	PyErr_Print();
 	PyErr_Clear();
+	pyg_unblock_threads();
 	return FALSE;
     }
 }
@@ -479,6 +523,9 @@ pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model, GtkTreeIter *
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -491,10 +538,12 @@ pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model, GtkTreeIter *
 	gint ret = PyInt_AsLong(py_ret);
 
 	Py_DECREF(py_ret);
+	pyg_unblock_threads();
 	return ret;
     } else {
 	PyErr_Print();
 	PyErr_Clear();
+	pyg_unblock_threads();
 	return 0;
     }
 }
@@ -508,6 +557,9 @@ pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model, GtkTreeIter  *
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -525,16 +577,19 @@ pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model, GtkTreeIter  *
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
+	    pyg_unblock_threads();
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
+	    pyg_unblock_threads();
 	    return FALSE;
 	}
     } else {
 	iter->user_data = NULL;
 	PyErr_Print();
 	PyErr_Clear();
+	pyg_unblock_threads();
 	return FALSE;
     }
 }
@@ -548,6 +603,9 @@ pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model, GtkTreeIter *iter
     g_return_val_if_fail(tree_model != NULL, FALSE);
     g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
+
+    pyg_block_threads();
+
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -565,16 +623,19 @@ pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model, GtkTreeIter *iter
 	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
 		Py_DECREF((PyObject *)iter->user_data);
 	    }
+	    pyg_unblock_threads();
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
 	    Py_DECREF(py_ret);
+	    pyg_unblock_threads();
 	    return FALSE;
 	}
     } else {
 	iter->user_data = NULL;
 	PyErr_Print();
 	PyErr_Clear();
+	pyg_unblock_threads();
 	return FALSE;
     }
 }
