@@ -216,7 +216,10 @@ pygobject_dealloc(PyGObject *self)
 	self->hasref = TRUE;
 	g_object_set_qdata_full(obj, pygobject_ownedref_key,
 				self, pyg_destroy_notify);
+	
+	pyg_unblock_threads();
 	g_object_unref(obj);
+	pyg_block_threads();
 
 	/* we ref the type, so subtype_dealloc() doesn't kill off our
          * instance's type. */
@@ -231,8 +234,11 @@ pygobject_dealloc(PyGObject *self)
 
 	return;
     }
-    if (obj && !self->hasref) /* don't unref the GObject if it owns us */
+    if (obj && !self->hasref) { /* don't unref the GObject if it owns us */
+	pyg_unblock_threads();
 	g_object_unref(obj);
+	pyg_block_threads();
+    }
 
     PyObject_ClearWeakRefs((PyObject *)self);
 
